@@ -29,6 +29,7 @@ from sunpy.data.test.waveunit import waveunitdir
 from sunpy.io import fits
 from sunpy.extern.six.moves import range
 from sunpy.extern.six.moves import configparser
+from sunpy.net import Fido, attrs as net_attrs
 
 import sunpy.data.test
 
@@ -36,6 +37,14 @@ testpath = sunpy.data.test.rootdir
 RHESSI_IMAGE = os.path.join(testpath, 'hsi_image_20101016_191218.fits')
 
 
+"""
+The 'hsi_image_20101016_191218.fits' file lies in the sunpy/data/test.
+RHESSI_IMAGE  = sunpy/data/test/hsi_image_20101016_191218.fits
+
+So, the tests in the database depends on the test under sunpy/data.
+"""
+
+
 @pytest.fixture
 def database_using_lrucache():
     return Database('sqlite:///:memory:', LRUCache, cache_size=3)
@@ -51,16 +60,31 @@ def database():
     return Database('sqlite:///:memory:')
 
 
+@pytest.fixture
+def fido_search_result():
+    # A search query with responses from all instruments
+    # No JSOC query
+    return Fido.search(
+        net_attrs.Time("2012/1/1", "2012/1/2"),
+        net_attrs.Instrument('lyra') | net_attrs.Instrument('eve') |
+        net_attrs.Instrument('XRS') | net_attrs.Instrument('noaa-indices') |
+        net_attrs.Instrument('noaa-predict') |
+        (net_attrs.Instrument('norh') & net_attrs.Wavelength(17*units.GHz)) |
+        net_attrs.Instrument('rhessi') |
+        (net_attrs.Instrument('EVE') & net_attrs.Level(0))
+        )
+
+
 @pytest.fixture
 def query_result():
-    return vso.VSOClient().query(
+    return vso.VSOClient().search(
         vso.attrs.Time('20130801T200000', '20130801T200030'),
         vso.attrs.Instrument('PLASTIC'))
 
 
 @pytest.fixture
 def download_qr():
-    return vso.VSOClient().query(
+    return vso.VSOClient().search(
         vso.attrs.Time('2012-03-29', '2012-03-29'),
         vso.attrs.Instrument('AIA'))
 
@@ -412,12 +436,93 @@ def test_add_already_existing_entry_ignore(database):
 
 @pytest.mark.online
 def test_add_entry_from_hek_qr(database):
-    hek_res = hek.HEKClient().query(
+    hek_res = hek.HEKClient().search(
         hek.attrs.Time('2011/08/09 07:23:56', '2011/08/09 07:24:00'),
         hek.attrs.EventType('FL'))
     assert len(database) == 0
     database.add_from_hek_query_result(hek_res)
-    assert len(database) == 1678
+    # This number loves to change, so we are just going to test that it's added
+    # *something*
+    assert len(database) > 1
+
+
+def num_entries_from_vso_query(db, query, path=None, file_pattern='',
+                               overwrite=False):
+    db.download_from_vso_query_result(
+        query, path=path, overwrite=overwrite)
+    fits_pattern = file_pattern
+    num_of_fits_headers = sum(
+        len(fits.get_header(file)) for file in glob.glob(fits_pattern))
+    return num_of_fits_headers
+
+
+@pytest.mark.online
+def test_vso_query_block_caching(database, download_qr, tmpdir):
+
+    assert len(database) == 0
+
+    # Download for all query response blocks and save the length
+    # of database in num_of_fits_headers
+    num_of_fits_headers = num_entries_from_vso_query(database, download_qr,
+                                                     path=str(tmpdir.join('{file}.fits')),
+                                                     file_pattern=str(tmpdir.join('*.fits')))
+
+    assert len(database) == num_of_fits_headers and len(database) > 0
+
+    # Emptying the database
+    database.clear()
+    database.commit()
+
+    # Only downloading for the first query response block
+    num_of_fits_headers_1 = num_entries_from_vso_query(database, download_qr[:1],
+                                                       path=str(tmpdir.join('{file}.type1')),
+                                                       file_pattern=str(tmpdir.join('*.type1')))
+
+    assert len(database) == num_of_fits_headers_1 and len(database) > 0
+
+    # Downloading for all query response blocks
+    num_of_fits_headers_2 = num_entries_from_vso_query(database, download_qr,
+                                                       path=str(tmpdir.join('{file}.type2')),
+                                                       file_pattern=str(tmpdir.join('*.type2')))
+
+    # Final length of the database should be the same as num_of_fits_headers.
+    # This is done to ensure that the first query response block's files weren't
+    # redownloaded. If they were redownloaded then length will be greater than
+    # num_of_fits_headers as new entries are added to the database in case of a
+    # download.
+
+    assert len(database) == num_of_fits_headers_1 + num_of_fits_headers_2
+    assert len(database) > 0
+
+    assert num_of_fits_headers_1 + num_of_fits_headers_2 == num_of_fits_headers
+
+
+@pytest.mark.online
+def test_vso_query_block_caching_with_overwrite_true_flag(database,
+                                                          download_qr, tmpdir):
+
+    assert len(database) == 0
+
+    # Download for all query response blocks and save the length
+    # of database in num_of_fits_headers
+
+    num_of_fits_headers = num_entries_from_vso_query(database, download_qr,
+                                                     path=str(tmpdir.join('{file}.fits')),
+                                                     file_pattern=str(tmpdir.join('*.fits')))
+
+    assert len(database) == num_of_fits_headers and len(database) > 0
+
+    # Only downloading for the first query response block with caching disabled
+
+    num_of_fits_headers_1 = num_entries_from_vso_query(database, download_qr[:1],
+                                                       path=str(tmpdir.join('{file}.type1')),
+                                                       file_pattern=str(tmpdir.join('*.type1')),
+                                                       overwrite=True)
+
+    # The files for the first query response block should be downloaded again
+    # Old entries should be deleted, so len(database) should not change
+    assert len(database) == num_of_fits_headers
+    assert len(database) > 0
 
 
 @pytest.mark.online
@@ -441,18 +546,18 @@ def test_download_from_qr(database, download_qr, tmpdir):
 def test_add_entry_from_qr(database, query_result):
     assert len(database) == 0
     database.add_from_vso_query_result(query_result)
-    assert len(database) == 10
+    assert len(database) == 16
     database.undo()
     assert len(database) == 0
     database.redo()
-    assert len(database) == 10
+    assert len(database) == 16
 
 
 @pytest.mark.online
 def test_add_entries_from_qr_duplicates(database, query_result):
     assert len(database) == 0
     database.add_from_vso_query_result(query_result)
-    assert len(database) == 10
+    assert len(database) == 16
     with pytest.raises(EntryAlreadyAddedError):
         database.add_from_vso_query_result(query_result)
 
@@ -461,9 +566,50 @@ def test_add_entries_from_qr_duplicates(database, query_result):
 def test_add_entries_from_qr_ignore_duplicates(database, query_result):
     assert len(database) == 0
     database.add_from_vso_query_result(query_result)
-    assert len(database) == 10
+    assert len(database) == 16
     database.add_from_vso_query_result(query_result, True)
-    assert len(database) == 20
+    assert len(database) == 32
+
+
+@pytest.mark.online
+def test_add_entry_fido_search_result(database, fido_search_result):
+    assert len(database) == 0
+    database.add_from_fido_search_result(fido_search_result)
+    assert len(database) == 65
+    database.undo()
+    assert len(database) == 0
+    database.redo()
+    assert len(database) == 65
+
+
+@pytest.mark.online
+def test_add_entries_from_fido_search_result_JSOC_client(database):
+    assert len(database) == 0
+    search_result = Fido.search(
+        net_attrs.jsoc.Time('2014-01-01T00:00:00', '2014-01-01T01:00:00'),
+        net_attrs.jsoc.Series('hmi.m_45s'),
+        net_attrs.jsoc.Notify("sunpy@sunpy.org")
+        )
+    with pytest.raises(ValueError):
+        database.add_from_fido_search_result(search_result)
+
+
+@pytest.mark.online
+def test_add_entries_from_fido_search_result_duplicates(database, fido_search_result):
+    assert len(database) == 0
+    database.add_from_fido_search_result(fido_search_result)
+    assert len(database) == 65
+    with pytest.raises(EntryAlreadyAddedError):
+        database.add_from_fido_search_result(fido_search_result)
+
+
+@pytest.mark.online
+def test_add_entries_from_fido_search_result_ignore_duplicates(database, fido_search_result):
+    assert len(database) == 0
+    database.add_from_fido_search_result(fido_search_result)
+    assert len(database) == 65
+    database.add_from_fido_search_result(fido_search_result, True)
+    assert len(database) == 2*65
 
 
 def test_add_fom_path(database):
@@ -500,6 +646,14 @@ def test_add_from_file(database):
         assert entry.fileid == fileid
 
 
+def test_add_from_file_hdu_index(database):
+    assert len(database) == 0
+    database.add_from_file(RHESSI_IMAGE)
+    assert len(database) == 4
+    for i, entry in enumerate(database):
+        assert entry.hdu_index == i
+
+
 def test_add_from_file_duplicates(database):
     database.add_from_file(RHESSI_IMAGE)
     with pytest.raises(EntryAlreadyAddedError):
@@ -697,12 +851,12 @@ def test_lfu_cache(database_using_lfucache):
 
 def test_query_missing_arg(database):
     with pytest.raises(TypeError):
-        database.query()
+        database.search()
 
 
 def test_query_unexpected_kwarg(database):
     with pytest.raises(TypeError):
-        database.query(attrs.Starred(), foo=42)
+        database.search(attrs.Starred(), foo=42)
 
 
 def test_query(filled_database):
@@ -710,7 +864,7 @@ def test_query(filled_database):
     foo.id = 1
     bar = Tag('bar')
     bar.id = 2
-    entries = filled_database.query(
+    entries = filled_database.search(
         attrs.Tag('foo') | attrs.Tag('bar'), sortby='id')
     assert len(entries) == 4
     assert entries == [
@@ -720,29 +874,24 @@ def test_query(filled_database):
         DatabaseEntry(id=10, tags=[bar])]
 
 
-def test_download_missing_arg(database):
+def test_fetch_missing_arg(database):
     with pytest.raises(TypeError):
-        database.download()
-
-
-def test_download_unexpected_kwarg(database):
-    with pytest.raises(TypeError):
-        database.download(vso.attrs.Source('SOHO'), foo=42)
+        database.fetch()
 
 
 @pytest.mark.online
-def test_download_empty_query_result(database, empty_query):
-    database.download(*empty_query)
+def test_fetch_empty_query_result(database, empty_query):
+    database.fetch(*empty_query)
     with pytest.raises(EmptyCommandStackError):
         database.undo()
     assert len(database) == 0
 
 
 @pytest.mark.online
-def test_download(database, download_query, tmpdir):
+def test_fetch(database, download_query, tmpdir):
     assert len(database) == 0
     database.default_waveunit = 'angstrom'
-    database.download(
+    database.fetch(
         *download_query, path=str(tmpdir.join('{file}.fits')), progress=True)
     fits_pattern = str(tmpdir.join('*.fits'))
     num_of_fits_headers = sum(
@@ -757,16 +906,18 @@ def test_download(database, download_query, tmpdir):
 
 
 @pytest.mark.online
-def test_download_duplicates(database, download_query, tmpdir):
+def test_fetch_duplicates(database, download_query, tmpdir):
     assert len(database) == 0
     database.default_waveunit = 'angstrom'
-    database.download(
+    database.fetch(
         *download_query, path=str(tmpdir.join('{file}.fits')), progress=True)
     assert len(database) == 4
     download_time = database[0].download_time
-    database.download(*download_query, path=str(tmpdir.join('{file}.fits')))
+    database.fetch(*download_query, path=str(tmpdir.join('{file}.fits')))
     assert len(database) == 4
-    assert database[0].download_time != download_time
+    # The old file should be untouched because of the query result block
+    # level caching
+    assert database[0].download_time == download_time
 
 
 def test_fetch_missing_arg(database):
@@ -774,11 +925,6 @@ def test_fetch_missing_arg(database):
         database.fetch()
 
 
-def test_fetch_unexpected_kwarg(database):
-    with pytest.raises(TypeError):
-        database.fetch(vso.attrs.Source('SOHO'), foo=42)
-
-
 @pytest.mark.online
 def test_fetch(database, download_query, tmpdir):
     assert len(database) == 0
@@ -814,13 +960,15 @@ def test_fetch_separate_filenames():
     db.fetch(*download_query, path=path)
 
     # Test
-    assert len(db) == 8
+    assert len(db) == 4
 
     dir_contents = os.listdir(tmp_test_dir)
     assert 'aia_lev1_335a_2012_08_05t00_00_02_62z_image_lev1.fits' in dir_contents
     assert 'aia_lev1_94a_2012_08_05t00_00_01_12z_image_lev1.fits' in dir_contents
-    assert os.path.isfile(os.path.join(tmp_test_dir, 'aia_lev1_335a_2012_08_05t00_00_02_62z_image_lev1.fits'))
-    assert os.path.isfile(os.path.join(tmp_test_dir, 'aia_lev1_94a_2012_08_05t00_00_01_12z_image_lev1.fits'))
+    assert os.path.isfile(os.path.join(
+        tmp_test_dir, 'aia_lev1_335a_2012_08_05t00_00_02_62z_image_lev1.fits'))
+    assert os.path.isfile(os.path.join(
+        tmp_test_dir, 'aia_lev1_94a_2012_08_05t00_00_01_12z_image_lev1.fits'))
 
     # Teardown
     shutil.rmtree(tmp_test_dir)
@@ -835,7 +983,7 @@ def test_disable_undo(database, download_query, tmpdir):
         db.commit()
         db.remove(entry)
         db.default_waveunit = 'angstrom'
-        db.download(*download_query, path=str(tmpdir.join('{file}.fits')))
+        db.fetch(*download_query, path=str(tmpdir.join('{file}.fits')))
         entry = db[0]
         db.tag(entry, 'foo', 'bar')
         db.remove_tag(entry, 'foo')
@@ -851,8 +999,9 @@ def test_disable_undo(database, download_query, tmpdir):
 
 @pytest.fixture
 def default_waveunit_database():
-    unit_database = Database('sqlite:///:memory:', default_waveunit = units.meter)
-    str_database = Database('sqlite:///:memory:', default_waveunit = "m")
+    unit_database = Database('sqlite:///:memory:',
+                             default_waveunit=units.meter)
+    str_database = Database('sqlite:///:memory:', default_waveunit="m")
     return unit_database, str_database
 
 
@@ -891,9 +1040,9 @@ def test_split_database(split_function_database, database):
     split_function_database, database = split_database(
         split_function_database, database, vso.attrs.Instrument('EIA'))
 
-    observed_source_entries = split_function_database.query(
+    observed_source_entries = split_function_database.search(
         vso.attrs.Provider('xyz'), sortby='id')
-    observed_destination_entries = database.query(vso.attrs.Provider('xyz'))
+    observed_destination_entries = database.search(vso.attrs.Provider('xyz'))
 
     assert observed_source_entries == [
         DatabaseEntry(id=1, instrument='RHESSI', provider='xyz'),
