@@ -8,6 +8,7 @@
 from __future__ import absolute_import, print_function
 
 import csv
+import socket
 from datetime import datetime
 from datetime import timedelta
 
@@ -16,14 +17,14 @@ import numpy as np
 from astropy.io import fits
 from astropy import units as u
 
-import sunpy.map
-import sunpy.sun.constants
-
 from sunpy.time import TimeRange, parse_time
 from sunpy.sun.sun import solar_semidiameter_angular_size
-from sunpy.sun.sun import sunearth_distance
+from sunpy.coordinates import get_sunearth_distance
+import sunpy.map
 
 from sunpy.extern.six.moves import urllib
+from sunpy.extern.six.moves.urllib.request import urlopen
+from sunpy.extern.six.moves.urllib.error import URLError
 
 __all__ = ['get_obssumm_dbase_file', 'parse_obssumm_dbase_file',
            'get_obssum_filename', 'get_obssumm_file', 'parse_obssumm_file',
@@ -35,7 +36,7 @@ grid_pitch = (4.52467, 7.85160, 13.5751, 23.5542, 40.7241, 70.5309, 122.164,
 grid_orientation = (3.53547, 2.75007, 3.53569, 2.74962, 3.92596, 2.35647,
                     0.786083, 0.00140674, 1.57147)
 
-data_servers = ('http://hesperia.gsfc.nasa.gov/hessidata/',
+data_servers = ('https://hesperia.gsfc.nasa.gov/hessidata/',
                 'http://hessi.ssl.berkeley.edu/hessidata/',
                 'http://soleil.i4ds.ch/hessidata/')
 
@@ -43,6 +44,20 @@ lc_linecolors = ('black', 'pink', 'green', 'blue', 'brown', 'red',
                  'navy', 'orange', 'green')
 
 
+def get_base_url():
+    """
+    Find the first mirror which is online
+    """
+
+    for server in data_servers:
+        try:
+            urlopen(server, timeout=1)
+        except (URLError, socket.timeout):
+            pass
+        else:
+            return server
+
+
 def get_obssumm_dbase_file(time_range):
     """
     Download the RHESSI observing summary database file. This file lists the
@@ -79,7 +94,10 @@ def get_obssumm_dbase_file(time_range):
     _time_range = TimeRange(time_range)
     data_location = 'dbase/'
 
-    url_root = data_servers[0] + data_location
+    if _time_range.start < parse_time("2002/02/01"):
+        raise ValueError("RHESSI summary files are not available for before 2002-02-01")
+
+    url_root = get_base_url() + data_location
     url = url_root + _time_range.start.strftime("hsi_obssumm_filedb_%Y%m.txt")
 
     f = urllib.request.urlretrieve(url)
@@ -155,8 +173,8 @@ def parse_obssumm_dbase_file(filename):
 def get_obssum_filename(time_range):
     """
     Download the RHESSI observing summary data from one of the RHESSI
-    servers, parses it, and returns the name of the obssumm file relevant for
-    the time range
+    servers, parses it, and returns the name of the obssumm files relevant for
+    the time range.
 
     Parameters
     ----------
@@ -165,13 +183,13 @@ def get_obssum_filename(time_range):
 
     Returns
     -------
-    out : string
-        Returns the filename of the observation summary file
+    out : list
+        Returns the filenames of the observation summary file
 
     Examples
     --------
     >>> import sunpy.instr.rhessi as rhessi
-    >>> rhessi.get_obssumm_filename(('2011/04/04', '2011/04/05'))   # doctest: +SKIP
+    >>> rhessi.get_obssum_filename(('2011/04/04', '2011/04/05'))   # doctest: +SKIP
 
     .. note::
         This API is currently limited to providing data from whole days only.
@@ -185,9 +203,11 @@ def get_obssum_filename(time_range):
     result = parse_obssumm_dbase_file(f[0])
     _time_range = TimeRange(time_range)
 
-    index_number = _time_range.start.day - 1
+    index_number_start = _time_range.start.day - 1
+    index_number_end = _time_range.end.day - 1
 
-    return data_servers[0] + data_location + result.get('filename')[index_number] + 's'
+    return [get_base_url() + data_location + filename
+            + 's' for filename in result.get('filename')[index_number_start:index_number_end]]
 
 
 def get_obssumm_file(time_range):
@@ -218,12 +238,8 @@ def get_obssumm_file(time_range):
     """
 
     time_range = TimeRange(time_range)
-    data_location = 'metadata/catalog/'
 
-    # TODO need to check which is the closest servers
-    url_root = data_servers[0] + data_location
-
-    url = url_root + get_obssum_filename(time_range)
+    url = get_obssum_filename(time_range)[0]
 
     print('Downloading file: ' + url)
     f = urllib.request.urlretrieve(url)
@@ -234,6 +250,8 @@ def get_obssumm_file(time_range):
 def parse_obssumm_file(filename):
     """
     Parse a RHESSI observation summary file.
+    Note: this is for the Lightcurve datatype only, the TimSeries uses the
+    parse_obssumm_hdulist(hdulist) method to enable implicit source detection.
 
     Parameters
     ----------
@@ -264,19 +282,61 @@ def parse_obssumm_file(filename):
               '50 - 100 keV', '100 - 300 keV', '300 - 800 keV', '800 - 7000 keV',
               '7000 - 20000 keV']
 
-    # the data stored in the fits file are "compressed" countrates stored as one byte
+    # The data stored in the FITS file are "compressed" countrates stored as
+    # one byte
     compressed_countrate = np.array(afits[6].data.field('countrate'))
 
     countrate = uncompress_countrate(compressed_countrate)
-    dim = np.array(countrate[:,0]).size
+    dim = np.array(countrate[:, 0]).size
 
-    time_array = [reference_time_ut + timedelta(0,time_interval_sec * a) for a in np.arange(dim)]
+    time_array = [reference_time_ut + timedelta(0, time_interval_sec * a) for a in np.arange(dim)]
 
-    #TODO generate the labels for the dict automatically from labels
+    # TODO generate the labels for the dict automatically from labels
     data = {'time': time_array, 'data': countrate, 'labels': labels}
 
     return header, data
 
+
+def parse_obssumm_hdulist(hdulist):
+    """
+    Parse a RHESSI observation summary file.
+
+    Parameters
+    ----------
+    hdulist : list
+        The HDU list from the fits file.
+
+    Returns
+    -------
+    out : `dict`
+        Returns a dictionary.
+
+    """
+    header = hdulist[0].header
+
+    reference_time_ut = parse_time(hdulist[5].data.field('UT_REF')[0])
+    time_interval_sec = hdulist[5].data.field('TIME_INTV')[0]
+    # label_unit = fits[5].data.field('DIM1_UNIT')[0]
+    # labels = fits[5].data.field('DIM1_IDS')
+    labels = ['3 - 6 keV', '6 - 12 keV', '12 - 25 keV', '25 - 50 keV',
+              '50 - 100 keV', '100 - 300 keV', '300 - 800 keV',
+              '800 - 7000 keV', '7000 - 20000 keV']
+
+    # The data stored in the fits file are "compressed" countrates stored as
+    # one byte
+    compressed_countrate = np.array(hdulist[6].data.field('countrate'))
+
+    countrate = uncompress_countrate(compressed_countrate)
+    dim = np.array(countrate[:, 0]).size
+
+    time_array = [reference_time_ut + timedelta(0, time_interval_sec * a) for a in np.arange(dim)]
+
+    #  TODO generate the labels for the dict automatically from labels
+    data = {'time': time_array, 'data': countrate, 'labels': labels}
+
+    return header, data
+
+
 def uncompress_countrate(compressed_countrate):
     """Convert the compressed count rate inside of observing summary file from
     a compressed byte to a true count rate
@@ -290,15 +350,24 @@ def uncompress_countrate(compressed_countrate):
     ----------
     Hsi_obs_summ_decompress.pro `<http://hesperia.gsfc.nasa.gov/ssw/hessi/idl/qlook_archive/hsi_obs_summ_decompress.pro>`_
     """
+
+    # Ensure uncompressed counts are between 0 and 255
+    if (compressed_countrate.min() < 0) or (compressed_countrate.max() > 255):
+        raise ValueError(
+            'Exepected uncompressed counts {} to in range 0-255'.format(compressed_countrate))
+
+    # TODO Must be a better way than creating entire lookup table on each call
     ll = np.arange(0, 16, 1)
     lkup = np.zeros(256, dtype='int')
-    sum = 0
+    _sum = 0
     for i in range(0, 16):
-        lkup[16 * i:16 * (i + 1)] = ll * 2 ** i + sum
+        lkup[16 * i:16 * (i + 1)] = ll * 2 ** i + _sum
         if i < 15:
-            sum = lkup[16 * (i + 1) - 1] + 2 ** i
+            _sum = lkup[16 * (i + 1) - 1] + 2 ** i
+
     return lkup[compressed_countrate]
 
+
 def hsi_linecolors():
     """Define discrete colors to use for RHESSI plots
 
@@ -314,7 +383,9 @@ def hsi_linecolors():
     ----------
     hsi_linecolors.pro `<http://hesperia.gsfc.nasa.gov/ssw/hessi/idl/gen/hsi_linecolors.pro`_
     """
-    return ('black', 'magenta', 'lime', 'cyan', 'y', 'red', 'blue', 'orange', 'olive')
+    return ('black', 'magenta', 'lime', 'cyan', 'y', 'red', 'blue', 'orange',
+            'olive')
+
 
 def _backproject(calibrated_event_list, detector=8, pixel_size=(1., 1.),
                  image_dim=(64, 64)):
@@ -344,10 +415,8 @@ def _backproject(calibrated_event_list, detector=8, pixel_size=(1., 1.),
     >>> import sunpy.instr.rhessi as rhessi
 
     """
-    afits = fits.open(calibrated_event_list)
-
-    #info_parameters = fits[2]
-    #detector_efficiency = info_parameters.data.field('cbe_det_eff$$REL')
+    # info_parameters = fits[2]
+    # detector_efficiency = info_parameters.data.field('cbe_det_eff$$REL')
 
     afits = fits.open(calibrated_event_list)
 
@@ -362,19 +431,22 @@ def _backproject(calibrated_event_list, detector=8, pixel_size=(1., 1.),
     grid_transmission = afits[fits_detector_index].data.field('gridtran')
     count = afits[fits_detector_index].data.field('count')
 
-    tempa = (np.arange(image_dim[0]*image_dim[1]) %  image_dim[0]) - (image_dim[0]-1)/2.
-    tempb = tempa.reshape(image_dim[0],image_dim[1]).transpose().reshape(image_dim[0]*image_dim[1])
+    tempa = (np.arange(image_dim[0] * image_dim[1]) % image_dim[0]) - (image_dim[0]-1)/2.
+    tempb = tempa.reshape(image_dim[0], image_dim[1]).transpose().reshape(image_dim[0]*image_dim[1])
 
-    pixel = np.array(list(zip(tempa,tempb)))*pixel_size[0]
-    phase_pixel = (2*np.pi/harm_ang_pitch)* ( np.outer(pixel[:,0], np.cos(this_roll_angle - grid_angle)) -
-                                              np.outer(pixel[:,1], np.sin(this_roll_angle - grid_angle))) + phase_map_center
+    pixel = np.array(list(zip(tempa, tempb)))*pixel_size[0]
+    phase_pixel = (2 * np.pi/harm_ang_pitch) *\
+                  (np.outer(pixel[:, 0], np.cos(this_roll_angle - grid_angle)) -
+                   np.outer(pixel[:, 1], np.sin(this_roll_angle - grid_angle))) + phase_map_center
     phase_modulation = np.cos(phase_pixel)
     gridmod = modamp * grid_transmission
-    probability_of_transmission = gridmod*phase_modulation + grid_transmission
+    probability_of_transmission = gridmod * phase_modulation + grid_transmission
     bproj_image = np.inner(probability_of_transmission, count).reshape(image_dim)
 
     return bproj_image
 
+
+@u.quantity_input(pixel_size=u.arcsec, image_dim=u.pix)
 def backprojection(calibrated_event_list, pixel_size=(1., 1.) * u.arcsec,
                    image_dim=(64, 64) * u.pix):
     """
@@ -407,60 +479,45 @@ def backprojection(calibrated_event_list, pixel_size=(1., 1.) * u.arcsec,
     >>> map.peek()   # doctest: +SKIP
 
     """
-    if not isinstance(pixel_size, u.Quantity):
-        raise ValueError("Must be astropy Quantity in arcseconds")
-    try:
-        pixel_size = pixel_size.to(u.arcsec)
-    except:
-        raise ValueError("'{0}' is not a valid pixel_size unit".format(pixel_size.unit))
-    if not (isinstance(image_dim, u.Quantity) and image_dim.unit == 'pix'):
-        raise ValueError("Must be astropy Quantity in pixels")
-
-    try:
-        import sunpy.data.sample
-    except ImportError:
-        import sunpy.data
-        sunpy.data.download_sample()
-    # This may need to be moved up to data from sample
-    calibrated_event_list = sunpy.data.sample.RHESSI_EVENT_LIST
+    pixel_size = pixel_size.to(u.arcsec)
+    image_dim = np.array(image_dim.to(u.pix).value, dtype=int)
 
     afits = fits.open(calibrated_event_list)
     info_parameters = afits[2]
     xyoffset = info_parameters.data.field('USED_XYOFFSET')[0]
     time_range = TimeRange(info_parameters.data.field('ABSOLUTE_TIME_RANGE')[0])
 
-    image = np.zeros(image_dim.value)
+    image = np.zeros(image_dim)
 
     # find out what detectors were used
     det_index_mask = afits[1].data.field('det_index_mask')[0]
     detector_list = (np.arange(9)+1) * np.array(det_index_mask)
     for detector in detector_list:
         if detector > 0:
-            image = image + _backproject(calibrated_event_list, detector=detector, pixel_size=pixel_size.value
-										 , image_dim=image_dim.value)
+            image = image + _backproject(calibrated_event_list, detector=detector,
+                                         pixel_size=pixel_size.value, image_dim=image_dim)
 
     dict_header = {
-        "DATE-OBS": time_range.center().strftime("%Y-%m-%d %H:%M:%S"),
+        "DATE-OBS": time_range.center.strftime("%Y-%m-%d %H:%M:%S"),
         "CDELT1": pixel_size[0],
         "NAXIS1": image_dim[0],
         "CRVAL1": xyoffset[0],
-        "CRPIX1": image_dim[0].value/2 + 0.5,
+        "CRPIX1": image_dim[0]/2 + 0.5,
         "CUNIT1": "arcsec",
         "CTYPE1": "HPLN-TAN",
         "CDELT2": pixel_size[1],
         "NAXIS2": image_dim[1],
         "CRVAL2": xyoffset[1],
-        "CRPIX2": image_dim[0].value/2 + 0.5,
+        "CRPIX2": image_dim[0]/2 + 0.5,
         "CUNIT2": "arcsec",
         "CTYPE2": "HPLT-TAN",
         "HGLT_OBS": 0,
         "HGLN_OBS": 0,
-        "RSUN_OBS": solar_semidiameter_angular_size(time_range.center()).value,
+        "RSUN_OBS": solar_semidiameter_angular_size(time_range.center).value,
         "RSUN_REF": sunpy.sun.constants.radius.value,
-        "DSUN_OBS": sunearth_distance(time_range.center()) * sunpy.sun.constants.au.value
+        "DSUN_OBS": get_sunearth_distance(time_range.center).value * sunpy.sun.constants.au.value
     }
 
-    header = sunpy.map.MapMeta(dict_header)
-    result_map = sunpy.map.Map(image, header)
+    result_map = sunpy.map.Map(image, dict_header)
 
     return result_map
