@@ -8,7 +8,9 @@ import pytest
 #pylint: disable=C0103,R0904,W0201,W0232,E1103
 import sunpy
 import sunpy.lightcurve
-from sunpy.data.test import (EVE_AVERAGES_CSV)
+from sunpy.data.test import get_test_filepath
+
+EVE_AVERAGES_CSV = get_test_filepath("EVE_He_II_304_averages.csv")
 
 @pytest.mark.online
 def test_eve():
