@@ -5,5 +5,5 @@ from __future__ import absolute_import
 The following link is the link to the registry, should it be necessary, this
 link can easily be replaced or swapped.
 """
-LINK = 'http://msslkz.mssl.ucl.ac.uk/helio_registry/viewResourceEntry_body.jsp?' \
-       'XML=true&IVORN=ivo%3A%2F%2Fhelio-vo.eu%2F'
\ No newline at end of file
+LINK = 'http://registry.helio-vo.eu/helio_registry/viewResourceEntry_body.jsp?'\
+       'XML=true&IVORN=ivo://helio-vo.eu'
